/*
 * Gaim osd plugin.
 *
 */

#include "internal.h"
#include "util.h"
#include "connection.h"
#include "conversation.h"
#include "core.h"
#include "debug.h"
#include "signals.h"
#include "prefs.h"
#include "version.h"
#include "xosd.h"
#include "gtkimhtml.h"
#include "gtkplugin.h"
#include "gtkutils.h"
#include <X11/Xlib.h>
#include <stdio.h>
#include <limits.h>
#include <time.h>
#include "../gaimnosd_config.h"

static GaimPlugin *my_plugin = NULL;
xosd *osd = NULL;
time_t last_print = 0;

/* options to customize */
static xosd_align osd_align = XOSD_left;
static xosd_pos osd_pos = XOSD_top;
static const char *osd_font_name = "-misc-fixed-iso10646-1";
static const char *osd_font_size = "20";
static const char *osd_color = "LightGoldenrod1";
static int osd_timeout = 15;
static int osd_shadow = 1;
static int osd_msgs = 0;
static int osd_lines = 1;

static int osd_xoffset = 20;
static int osd_yoffset = 20;

static char **x_font_names = NULL;
static char *x_font_sizes[] = {
  "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
  "24", "26", "28", "32", "36", "40", "48", "56", "64", "72", NULL };


static void
osd_fill_fonts ()
{
  Display *disp;

  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_fill_fonts\n");
  disp = XOpenDisplay (NULL);
  char **fnts;
  int cnt = 0, i, i2;

  if (disp)
    {
      fnts = XListFonts (disp, "*", INT_MAX, &cnt);

      if (fnts && cnt)
	{
          x_font_names = (char **) g_malloc ((cnt + 1) * sizeof (char *));
	  x_font_names [cnt] = NULL;
	  for (i = 0; i < cnt; i++)
	    {
	      char buffer [1000];
	      char *ptr, *old_ptr;

	      x_font_names [i] = (char *) 1;

	      ptr = strchr (fnts[i] + 1, '-');
	      if (!ptr) continue;
	      ptr = strchr (ptr + 1, '-');
	      if (!ptr) continue;
	      strncpy (buffer, fnts[i] + 1, ptr - fnts[i] - 1);
	      buffer[ptr - fnts[i] - 1] = 0;

	      old_ptr = ptr = strrchr (fnts[i], '-');
	      if (!ptr) continue;
	      *old_ptr = 0;
	      ptr = strrchr (fnts[i], '-');
	      *old_ptr = '-';
	      if (!ptr) continue;
	      strcat (buffer, ptr);

	      for (i2 = 0; i2 < i; i2++)
		{
		  if (x_font_names[i2] != (char *) 1 && !strcmp (x_font_names[i2], buffer))
		    break;
		}
	      if (i2 < i)
		continue;
	      x_font_names[i] = g_malloc (strlen (buffer) + 1);
	      strcpy (x_font_names[i], buffer);
	    }

	  XFreeFontNames (fnts);
	}
      else
	{
	  gaim_debug(GAIM_DEBUG_INFO, "osd", "no X fonts\n");
	}
      XCloseDisplay (disp);
    }
  else
    {
      gaim_debug(GAIM_DEBUG_INFO, "osd", "no X display\n");
    }
}


static void
osd_init_prefs (void)
{
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_init_prefs\n");
  gaim_prefs_add_none ("/plugins/gtk");
  gaim_prefs_add_none ("/plugins/gtk/X11");
  gaim_prefs_add_none ("/plugins/gtk/X11/gaimnnosd");

  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/align", (int) osd_align);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/position", (int) osd_pos);
  gaim_prefs_add_string ("/plugins/gtk/X11/gaimnnosd/font_name", osd_font_name);
  gaim_prefs_add_string ("/plugins/gtk/X11/gaimnnosd/font_size", osd_font_size);
  gaim_prefs_add_string ("/plugins/gtk/X11/gaimnnosd/color", osd_color);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/timeout", osd_timeout);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/shadow", osd_shadow);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/lines", osd_lines);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/xoffset", osd_xoffset);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/yoffset", osd_yoffset);
  gaim_prefs_add_int ("/plugins/gtk/X11/gaimnnosd/msgs", osd_msgs);
}


static void
osd_get_prefs (void)
{
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_get_prefs\n");
  osd_align = (xosd_align) gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/align");
  osd_pos = (xosd_pos) gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/position");
  osd_font_name = gaim_prefs_get_string ("/plugins/gtk/X11/gaimnnosd/font_name");
  osd_font_size = gaim_prefs_get_string ("/plugins/gtk/X11/gaimnnosd/font_size");
  osd_color = gaim_prefs_get_string ("/plugins/gtk/X11/gaimnnosd/color");
  osd_timeout = gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/timeout");
  osd_shadow = gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/shadow");
  osd_lines = gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/lines");
  osd_xoffset = gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/xoffset");
  osd_yoffset = gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/yoffset");
  osd_msgs = gaim_prefs_get_int ("/plugins/gtk/X11/gaimnnosd/msgs");
}


static void
osd_set_font ()
{
  char buffer[1000];
  char *inc_point;

  inc_point = strchr (osd_font_name, '-');
  if (!inc_point) return;
  inc_point = strchr (inc_point + 1, '-');
  if (!inc_point) return;
  *inc_point = 0;
  sprintf (buffer, "-%s-*-*-*-*-%s-*-*-*-*-*-%s", osd_font_name, osd_font_size, inc_point + 1);
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_set_font : buf=%s\n", buffer);
  xosd_set_font (osd, buffer);
  *inc_point = '-';
}


static void
osd_init ()
{
  if (!osd)
    {
      osd_get_prefs ();
      osd = xosd_create (osd_lines);
      xosd_set_align (osd, osd_align);
      xosd_set_pos (osd, osd_pos);
      osd_set_font();
      xosd_set_colour(osd, osd_color);
      xosd_set_timeout(osd, osd_timeout);
      xosd_set_shadow_offset(osd, osd_shadow);
      xosd_set_horizontal_offset (osd, osd_xoffset);
      xosd_set_vertical_offset (osd, osd_yoffset);
    }
}


static void
osd_print (const char *text)
{
  time_t now;

  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_print\n");
  osd_init ();
  time(&now);
  if (last_print + osd_timeout > now)
    {
      xosd_scroll(osd, 1);
    }
  else
    {
      xosd_scroll(osd, osd_lines);
    }
  xosd_display (osd, osd_lines-1, XOSD_string, gaim_markup_strip_html (text));
  last_print = now;
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_print done\n");
}


void
osd_set_prefs (void)
{
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_set_prefs\n");
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/align", (int) osd_align);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/position", (int) osd_pos);
  gaim_prefs_set_string ("/plugins/gtk/X11/gaimnnosd/font_name", osd_font_name);
  gaim_prefs_set_string ("/plugins/gtk/X11/gaimnnosd/font_size", osd_font_size);
  gaim_prefs_set_string ("/plugins/gtk/X11/gaimnnosd/color", osd_color);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/timeout", osd_timeout);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/shadow", osd_shadow);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/lines", osd_lines);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/xoffset", osd_xoffset);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/yoffset", osd_yoffset);
  gaim_prefs_set_int ("/plugins/gtk/X11/gaimnnosd/msgs", osd_msgs);

  if (osd)
    {
      xosd_destroy (osd);
      osd = NULL;
    }
  
  osd_init ();

  if (osd)
    {
      int i;

      xosd_set_align (osd, osd_align);
      xosd_set_pos (osd, osd_pos);
      osd_set_font();
      xosd_set_colour(osd, osd_color);
      xosd_set_timeout(osd, osd_timeout);
      xosd_set_shadow_offset(osd, osd_shadow);
      xosd_set_horizontal_offset (osd, osd_xoffset);
      xosd_set_vertical_offset (osd, osd_yoffset);

      for (i=0; i < osd_lines; i++)
        {
          osd_print (_("Sample text message sample text message"));
        }
    }
}

#define GAIM_BUDDY_DISP_NAME(buddy) \
  ((buddy)->alias ? (buddy)->alias : ((buddy)->server_alias ? (buddy)->server_alias : (buddy)->name));


static void
osd_notify (GaimBuddy *buddy, const char *buddy_action)
{
  char text_buffer[1000];
  char *disp_name = GAIM_BUDDY_DISP_NAME (buddy);
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_notify\n");
  sprintf (text_buffer, "%.300s %.300s", disp_name, buddy_action);
  osd_print (text_buffer);
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_notify done\n");
}

static void
osd_notify_txt (const char *format, ...)
{
  char text_buffer[1000];
  va_list lst;
  va_start (lst, format);
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_notify_txt\n");
  vsnprintf (text_buffer, sizeof (text_buffer), format, lst);
  va_end (lst);
  text_buffer[sizeof (text_buffer) - 1] = 0;
  osd_print (text_buffer);
  gaim_debug(GAIM_DEBUG_INFO, "osd", "osd_notify_txt done\n");
}

static void
buddy_away_cb(GaimBuddy *buddy, void *data)
{
	gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_away_cb\n");
	osd_notify (buddy, _("is away"));
}

static void
buddy_back_cb(GaimBuddy *buddy, void *data)
{
	gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_back_cb\n");
	osd_notify (buddy, _("is back"));
}

static void
buddy_idle_cb(GaimBuddy *buddy, void *data)
{
	gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_idle_cb\n");
	osd_notify (buddy, _("is idle"));
}

static void
buddy_unidle_cb(GaimBuddy *buddy, void *data)
{
	gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_unidle_cb\n");
	osd_notify (buddy, _("is not idle"));
}

static void
buddy_signed_on_cb(GaimBuddy *buddy, void *data)
{
	gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_signed_on_cb\n");
	osd_notify (buddy, _("signed on"));
}

static void
buddy_signed_off_cb(GaimBuddy *buddy, void *data)
{
	gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_signed_off_cb\n");
	osd_notify (buddy, _("signed off"));
}

static void
buddy_msg_received_cb(GaimAccount *acct, char *sender, char *message,
    GaimConversation *conv, int *flags)
{
  gaim_debug(GAIM_DEBUG_INFO, "osd", "buddy_msg_received_cb\n");
  if (osd_msgs)
    {
      char *disp_name = sender;
      GaimBuddy *buddy = gaim_find_buddy (acct, sender);
      if (buddy)
	disp_name = GAIM_BUDDY_DISP_NAME (buddy);

      osd_notify_txt (_("&lt;%.100s&gt; : %.300s"), disp_name, message);
    }
}

static gboolean
plugin_load(GaimPlugin *plugin)
{
	void *blist_handle = NULL;
	my_plugin = plugin;

	gaim_debug(GAIM_DEBUG_INFO, "osd", "plugin_load start\n");
	blist_handle = gaim_blist_get_handle();
	/* Buddy List subsystem signals */
	gaim_signal_connect(blist_handle, "buddy-away",
	    plugin, GAIM_CALLBACK(buddy_away_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-back",
	    plugin, GAIM_CALLBACK(buddy_back_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-idle",
	    plugin, GAIM_CALLBACK(buddy_idle_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-unidle",
	    plugin, GAIM_CALLBACK(buddy_unidle_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-signed-on",
	    plugin, GAIM_CALLBACK(buddy_signed_on_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-signed-off",
	    plugin, GAIM_CALLBACK(buddy_signed_off_cb), NULL);
 	gaim_signal_connect(gaim_conversations_get_handle(), "received-im-msg",
 	    plugin, GAIM_CALLBACK(buddy_msg_received_cb), NULL);
	/*gaim_signal_connect (gaim_connections_get_handle(), "signed-on", GAIM_CALLBACK(signed_on), NULL);*/

	gaim_debug(GAIM_DEBUG_INFO, "osd", "plugin_load done\n");
	return TRUE;
}

static gboolean
plugin_unload(GaimPlugin *plugin)
{
  xosd_wait_until_no_display(osd);
  xosd_destroy (osd);
  osd = NULL;
  return TRUE;
}

GtkWidget *g_osd_font_name = NULL;
GtkWidget *g_osd_font_size = NULL;
GtkWidget *g_osd_color = NULL;
GtkWidget *g_osd_align = NULL;
GtkWidget *g_osd_xoffset = NULL;
GtkWidget *g_osd_pos = NULL;
GtkWidget *g_osd_yoffset = NULL;
GtkWidget *g_osd_timeout = NULL;
GtkWidget *g_osd_shadow = NULL;
GtkWidget *g_osd_lines = NULL;
GtkWidget *g_osd_msgs = NULL;
GtkTooltips *g_tooltips = NULL;

static void
osd_set_values(GtkWidget *button, GtkWidget *data)
{
  GdkColor c;
  static char buf[20];

  gtk_color_selection_get_current_color (GTK_COLOR_SELECTION (g_osd_color), &c);
  g_snprintf(buf, sizeof(buf), "#%04x%04x%04x", c.red, c.green, c.blue);
  osd_color = buf;

  osd_font_name = g_object_get_data (G_OBJECT (gtk_menu_get_active (
	GTK_MENU (gtk_option_menu_get_menu (
	    GTK_OPTION_MENU (g_osd_font_name))))), "val");
  gaim_debug(GAIM_DEBUG_INFO, "osd", "font_name=%s\n", osd_font_name);

  osd_font_size = g_object_get_data (G_OBJECT (gtk_menu_get_active (
	GTK_MENU (gtk_option_menu_get_menu (
	    GTK_OPTION_MENU (g_osd_font_size))))), "val");
  gaim_debug(GAIM_DEBUG_INFO, "osd", "font_name=%s\n", osd_font_size);

  osd_align = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (gtk_menu_get_active (
	GTK_MENU (gtk_option_menu_get_menu (
	    GTK_OPTION_MENU (g_osd_align))))), "val"));

  osd_pos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (gtk_menu_get_active (
	GTK_MENU (gtk_option_menu_get_menu (
	    GTK_OPTION_MENU (g_osd_pos))))), "val"));

  osd_timeout = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (g_osd_timeout));
  osd_shadow = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (g_osd_shadow));
  osd_lines = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (g_osd_lines));
  osd_xoffset = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (g_osd_xoffset));
  osd_yoffset = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (g_osd_yoffset));
  osd_msgs = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (g_osd_msgs)) == TRUE ? 1 : 0;

  osd_set_prefs ();
}


static void
osd_read_values ()
{
  GdkColor c;
  GtkWidget *align, *pos, *item, *font_name, *font_size;
  int i, inx, active_font_inx = 0, active_font_size_inx = 0;

  osd_get_prefs ();

  gdk_color_parse (osd_color, &c);
  gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (g_osd_color), &c);
  gtk_color_selection_set_previous_color (GTK_COLOR_SELECTION (g_osd_color), &c);

  font_name = gtk_menu_new ();
  if (x_font_names)
    {
      for (i = 0, inx = 0; x_font_names[i]; i++)
	{
	  if (x_font_names[i] != (char *) 1)
	    {
	      item = gtk_menu_item_new_with_label (x_font_names[i]);
	      g_object_set_data (G_OBJECT (item), "val", x_font_names[i]);
	      gtk_menu_append (GTK_MENU (font_name), item);
	      if (!strcmp (osd_font_name, x_font_names[i]))
		active_font_inx = inx;
	      inx ++;
	    }
	}
      gtk_menu_set_active (GTK_MENU (font_name), active_font_inx);
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (g_osd_font_name), font_name);

  font_size = gtk_menu_new ();
  for (i = 0; x_font_sizes[i]; i++)
    {
      item = gtk_menu_item_new_with_label (x_font_sizes[i]);
      g_object_set_data (G_OBJECT (item), "val", x_font_sizes[i]);
      gtk_menu_append (GTK_MENU (font_size), item);
      if (!strcmp (osd_font_size, x_font_sizes[i]))
	active_font_size_inx = i;
    }
  gtk_menu_set_active (GTK_MENU (font_size), active_font_size_inx);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (g_osd_font_size), font_size);

  align = gtk_menu_new ();
  item = gtk_menu_item_new_with_label (_("Left"));
  g_object_set_data (G_OBJECT (item), "val", GINT_TO_POINTER (XOSD_left));
  gtk_menu_append (GTK_MENU (align), item);
  item = gtk_menu_item_new_with_label (_("Center"));
  g_object_set_data (G_OBJECT (item), "val", GINT_TO_POINTER (XOSD_center));
  gtk_menu_append (GTK_MENU (align), item);
  item = gtk_menu_item_new_with_label (_("Right"));
  g_object_set_data (G_OBJECT (item), "val", GINT_TO_POINTER (XOSD_right));
  gtk_menu_append (GTK_MENU (align), item);
  gtk_menu_set_active (GTK_MENU (align), osd_align);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (g_osd_align), align);

  pos = gtk_menu_new ();
  item = gtk_menu_item_new_with_label (_("Top"));
  g_object_set_data (G_OBJECT (item), "val", GINT_TO_POINTER (XOSD_top));
  gtk_menu_append (GTK_MENU (pos), item);
  item = gtk_menu_item_new_with_label (_("Bottom"));
  g_object_set_data (G_OBJECT (item), "val", GINT_TO_POINTER (XOSD_bottom));
  gtk_menu_append (GTK_MENU (pos), item);
  item = gtk_menu_item_new_with_label (_("Middle"));
  g_object_set_data (G_OBJECT (item), "val", GINT_TO_POINTER (XOSD_middle));
  gtk_menu_append (GTK_MENU (pos), item);
  gtk_menu_set_active (GTK_MENU (pos), osd_pos);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (g_osd_pos), pos);

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (g_osd_timeout), osd_timeout);

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (g_osd_shadow), osd_shadow);

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (g_osd_lines), osd_lines);

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (g_osd_xoffset), osd_xoffset);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (g_osd_yoffset), osd_yoffset);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (g_osd_msgs), osd_msgs != 0 ? TRUE : FALSE);
}

static GtkWidget *
osd_get_config_frame(GaimPlugin *plugin)
{
	GtkWidget *ret;
	GtkWidget *frame, *label;
	GtkWidget *vbox, *table, *hbox;
	GtkWidget *button;
	GtkAttachOptions opts = GTK_EXPAND | GTK_SHRINK | GTK_FILL;

	g_tooltips = gtk_tooltips_new ();

	ret = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(ret), 12);

	frame = gaim_gtk_make_frame(ret, _("OnScreen display properties"));

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);

	table = gtk_table_new (9, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 5);

	label = gtk_label_new(_("Font"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, opts, opts, 3, 3);
	hbox = gtk_hbox_new (FALSE, 5);
	g_osd_font_name = gtk_option_menu_new ();
	gtk_box_pack_start(GTK_BOX(hbox), g_osd_font_name, FALSE, FALSE, 5);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_font_name,
	    _("The font used to display the notification messages"), "");
	g_osd_font_size = gtk_option_menu_new ();
	gtk_box_pack_start(GTK_BOX(hbox), g_osd_font_size, FALSE, FALSE, 5);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_font_size,
	    _("The font size used to display the notification messages (in points)"), "");

	gtk_table_attach(GTK_TABLE(table), hbox, 1, 3, 0, 1, opts, opts, 3, 3);

	label = gtk_label_new(_("Color"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, opts, opts, 3, 3);
	g_osd_color = gtk_color_selection_new ();
	gtk_table_attach(GTK_TABLE(table), g_osd_color, 1, 3, 1, 2, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_color,
	    _("The foreground color used to display the notification messages"), "");

	label = gtk_label_new(_("Align"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, opts, opts, 3, 3);
	g_osd_align = gtk_option_menu_new ();
	gtk_table_attach(GTK_TABLE(table), g_osd_align, 1, 3, 2, 3, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_align,
	    _("How the notification messages are aligned horizontally"), "");

	label = gtk_label_new(_("X Offset"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, opts, opts, 3, 3);
	g_osd_xoffset = gtk_spin_button_new_with_range (1, 2000, 1);
	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (g_osd_xoffset), 0);
	gtk_table_attach(GTK_TABLE(table), g_osd_xoffset, 1, 3, 3, 4, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_xoffset,
	    _("Horizontal offset of notification message from the allignment point"), "");

	label = gtk_label_new(_("Position"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, opts, opts, 3, 3);
	g_osd_pos = gtk_option_menu_new ();
	gtk_table_attach(GTK_TABLE(table), g_osd_pos, 1, 3, 4, 5, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_align,
	    _("How the notification messages are aligned vertically"), "");

	label = gtk_label_new(_("Y Offset"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6, opts, opts, 3, 3);
	g_osd_yoffset = gtk_spin_button_new_with_range (1, 2000, 1);
	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (g_osd_yoffset), 0);
	gtk_table_attach(GTK_TABLE(table), g_osd_yoffset, 1, 3, 5, 6, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_yoffset,
	    _("Vertical offset of notification message from the allignment point"), "");

	label = gtk_label_new(_("Timeout (secs)"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7, opts, opts, 3, 3);
	g_osd_timeout = gtk_spin_button_new_with_range (1, 100, 1);
	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (g_osd_timeout), 0);
	gtk_table_attach(GTK_TABLE(table), g_osd_timeout, 1, 3, 6, 7, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_timeout,
	    _("How long to keep the messages on the screen"), "");

	label = gtk_label_new(_("Shadow"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 7, 8, opts, opts, 3, 3);
	g_osd_shadow = gtk_spin_button_new_with_range (1, 10, 1);
	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (g_osd_shadow), 0);
	gtk_table_attach(GTK_TABLE(table), g_osd_shadow, 1, 3, 7, 8, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_shadow,
	    _("The size of the shadow thrown by letters used to display the notification messages"), "");

        label = gtk_label_new(_("Lines"));
        gtk_table_attach(GTK_TABLE(table), label, 0, 1, 8, 9, opts, opts, 3, 3);
        g_osd_lines = gtk_spin_button_new_with_range (1, 10, 1);
        gtk_spin_button_set_digits (GTK_SPIN_BUTTON (g_osd_lines), 0);
        gtk_table_attach(GTK_TABLE(table), g_osd_lines, 1, 3, 8, 9, opts, opts, 3, 3);
        gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_lines,
            _("The number of lines that can be displayed when several messages appear at the same time"), "");

	g_osd_msgs = gtk_check_button_new_with_label (_("Show incoming messages"));
	gtk_table_attach(GTK_TABLE(table), g_osd_msgs, 0, 3, 9, 10, opts, opts, 3, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), g_osd_msgs,
	    _("If on will also show the incomming messages as OSD notifications"), "");

	button = gtk_button_new_with_mnemonic(_("_Set"));
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(osd_set_values), NULL);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (g_tooltips), button,
	    _("Writes & activates the new settings"), "");

	osd_read_values ();

	gtk_widget_show_all(ret);
	return ret;
}


static GaimGtkPluginUiInfo ui_info =
{
	osd_get_config_frame
};

static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_STANDARD,                             /**< type           */
	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
	0,                                                /**< flags          */
	NULL,                                             /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                            /**< priority       */

	"gaimnosd",	                                  /**< id             */
	N_("OSD Notification"),                           /**< name           */
	GAIMNOSD_VERSION,                                 /**< version        */
	                                                  /**  summary        */
	N_("Provides OSD messages."),
	                                                  /**  description    */
	N_("Provides OSD messages."),
	"georgekodinov@users.sourceforge.net",
	                                                  /**< author         */
	"gaimnosd.sourceforge.net",                       /**< homepage       */

	plugin_load,                                      /**< load           */
	plugin_unload,                                    /**< unload         */
	NULL,                                             /**< destroy        */

	&ui_info,                                         /**< ui_info        */
	NULL                                              /**< extra_info     */
};


static void
init_plugin(GaimPlugin *plugin)
{
  osd_fill_fonts ();
  osd_init_prefs ();
}


GAIM_INIT_PLUGIN(osd, init_plugin, info)
